<?php

class paypalIPN extends galOps {

  public $paypal_post_vars;
  public $paypal_response;
  public $url_string;
  public $timeout;
  public $error_log_file = 'logs/paypal.log';
  public $fields = [];
  public $endpoints = array(
    'sandbox' => 'https://ipnpb.sandbox.paypal.com/cgi-bin/webscr',
    'live' => 'https://ipnpb.paypal.com/cgi-bin/webscr'
  );
  public $sendpoints = array(
    'sandbox' => 'https://www.sandbox.paypal.com/cgi-bin/webscr',
    'live' => 'https://www.paypal.com/cgi-bin/webscr'
  );
  public $log, $sandbox;

  // Class Constructor..
  public function __construct($settings) {
    $this->paypal_post_vars = $_POST;
    $this->timeout          = 120;
    $this->log              = $settings->ppLog;
    $this->sandbox          = $settings->ppSandbox;
    $this->url_string       = ($settings->ppSandbox ? $this->sendpoints['sandbox'] : $this->sendpoints['live']);
  }

  public function postStream($str = []) {
    $arr = [];
    if ($str['stream']) {
      foreach (explode($str['del'][0], $str['stream']) AS $var) {
        $params = explode($str['del'][1], $var);
        if (isset($params[0], $params[1])) {
          $arr[$params[0]] = urldecode($params[1]);
        }
      }
    }
    return $arr;
  }

  // Send data back to Paypal..
  public function send_response() {
    $inc_post = file_get_contents('php://input');
    if ($inc_post) {
      $_POST = $this->postStream(array(
        'stream' => $inc_post,
        'del' => array('&', '=')
      ));
    }
    $ch = curl_init(($this->sandbox ? $this->endpoints['sandbox'] : $this->endpoints['live']));
    curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
    curl_setopt($ch, CURLOPT_POST, 1);
    curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($_POST) . '&cmd=_notify-validate');
    curl_setopt($ch, CURLOPT_USERAGENT, 'PHP-Callback-Verification-Script');
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 1);
    curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 2);
    curl_setopt($ch, CURLOPT_FORBID_REUSE, 1);
    curl_setopt($ch, CURLOPT_HTTPHEADER, array('Connection: Close'));
    curl_setopt($ch, CURLOPT_CAINFO, __dir__ . '/certs/cacert.pem');
    curl_setopt($ch, CURLOPT_TIMEOUT, 120);
    $this->paypal_response = curl_exec($ch);
    curl_close($ch);
  }

  // Was payment verified..
  public function is_verified() {
    return (strpos(strtolower($this->paypal_response), 'verified') !== false || strpos(strtolower($this->paypal_response), 'verified') > 0 ? true : false);
  }

  // Returns payment status..
  public function get_payment_status() {
    return $this->paypal_post_vars['payment_status'];
  }

  // Logs error messages..
  public function error_out($msg) {
    $date    = date("D M j G:i:s T Y", time());
    $message = '';
    $message .= $this->define_newline() . $this->define_newline() . $msg . $this->define_newline() . $this->define_newline();
    reset($this->paypal_post_vars);
    foreach ($this->paypal_post_vars AS $key => $value) {
      $message .= $key . ':' . " \t" . $value . $this->define_newline();
    }
    $message .= $this->define_newline() . $this->define_newline() . $this->url_string . $this->define_newline() . $this->define_newline() . $this->paypal_response . $this->define_newline() . $this->define_newline();
    if ($this->log) {
      file_put_contents($this->error_log_file, trim($message), FILE_APPEND);
    }
  }

  // Assigns field array..
  public function add_field($field, $value) {
    $this->fields[$field] = $value;
  }

  // Loads processing screen..
  public function loadHiddenFields() {
    $html = '<form method="post" name="form" action="' . $this->url_string . '">' . $this->define_newline();
    foreach ($this->fields AS $name => $value) {
      $html .= '<input type="hidden" name="' . $name . '" value="' . $this->strip_slashes($value) . '">' . $this->define_newline();
    }
    $html .= '</form>';
    return $html;
  }

}

?>